<?php

namespace QUI\ERP\Payments\Stripe\PaymentMethods;

use Exception;
use QUI;
use QUI\ERP\Accounting\Invoice\Invoice;
use QUI\ERP\Accounting\Invoice\InvoiceTemporary;
use QUI\ERP\Accounting\Invoice\InvoiceView;
use QUI\ERP\Accounting\Payments\Payments;
use QUI\ERP\Order\AbstractOrder;
use QUI\ERP\Payments\Stripe\AbstractBasePayment;
use Stripe\Exception\ApiErrorException;
use Stripe\PaymentIntent as StripePaymentIntent;
use QUI\ERP\Payments\Stripe\Utils;

/**
 * Class Card
 *
 * Stripe payment with credit card
 */
class Card extends AbstractBasePayment
{
    /**
     * @return string
     */
    public function getTitle(): string
    {
        return $this->getLocale()->get('quiqqer/payment-stripe', 'payment.Card.title');
    }

    /**
     * @return string
     */
    public function getDescription(): string
    {
        return $this->getLocale()->get('quiqqer/payment-stripe', 'payment.Card.description');
    }

    /**
     * Get title for frontend
     *
     * @return string
     */
    public function getFrontendTitle(): string
    {
        return $this->getLocale()->get('quiqqer/payment-stripe', 'payment.frontend.Card.title');
    }

    /**
     * Get description for frontend
     *
     * @return string
     */
    public function getFrontendDescription(): string
    {
        return $this->getLocale()->get('quiqqer/payment-stripe', 'payment.frontend.Card.description');
    }

    /**
     * Get title for the Payment step (OrderProcess)
     *
     * @return string
     */
    public function getPaymentStepTitle(): string
    {
        return $this->getLocale()->get('quiqqer/payment-stripe', 'payment.PaymentStep.title.Card');
    }

    /**
     * Get description step for the Payment step (OrderProcess)
     *
     * @return string
     */
    public function getPaymentStepInfo(): string
    {
        return $this->getLocale()->get('quiqqer/payment-stripe', 'payment.PaymentStep.info.Card');
    }

    /**
     * Return the payment icon (the URL path)
     * Can be overwritten
     *
     * @return string
     */
    public function getIcon(): string
    {
        return Payments::getInstance()->getHost() .
            URL_OPT_DIR .
            'quiqqer/payment-stripe/bin/images/Payment_Card.png';
    }

    /**
     * Get type string of Stripe PaymentMethod
     *
     * @return string
     */
    public function getPaymentMethodType(): string
    {
        return \Stripe\Card::OBJECT_NAME;
    }

    /**
     * Return attributes for creating a PaymentIntent
     *
     * @param AbstractOrder $Order
     * @param string $paymentMethodId - Stripe PaymentMethod ID
     * @return StripePaymentIntent
     *
     * @throws ApiErrorException
     * @throws QUI\Exception
     */
    protected function createPaymentIntentForOrder(AbstractOrder $Order, string $paymentMethodId): StripePaymentIntent
    {
        return StripePaymentIntent::create([
            'payment_method' => $paymentMethodId,
            'amount' => Utils::getCentAmount($Order),
            'currency' => mb_strtolower($Order->getCurrency()->getCode()),
//            'confirmation_method' => 'manual',
            'confirm' => true,
            'description' => Utils::getPaymentDescriptionForOrder($Order),
            'automatic_payment_methods' => [
                'enabled' => true,
                'allow_redirects' => 'never'
            ]
        ]);
    }

    /**
     * Return the extra text for the invoice
     *
     * @param Invoice|InvoiceTemporary|InvoiceView $Invoice |InvoiceView $Invoice
     * @return mixed
     */
    public function getInvoiceInformationText(Invoice|InvoiceTemporary|InvoiceView $Invoice): string
    {
        try {
            return $Invoice->getCustomer()->getLocale()->get(
                'quiqqer/payment-stripe',
                'additional_invoice_text.Card'
            );
        } catch (Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return '';
        }
    }
}
